;
;   64 bit Menuet Mail   
;
;   Compile with FASM 1.60 or above
;
;   (c) V.Turjanmaa
;

use64

    org   0x0

    db    'MENUET64'              ; Header identifier
    dq    0x01                    ; Version
    dq    START                   ; Start of code
    dq    I_END                   ; Size of image
    dq    0x200000                ; Memory for app
    dq    0x1ffff0                ; Rsp
    dq    0x00                    ; Prm 
    dq    0x00                    ; Icon


; image_end - from & subject - 80 step 
; 0xd0000   - email fetch server message
; 0xe0000   - server messages ok/err
; 0xf0000   - tmp header read - tmp mail read - tmp topwindow check
; 0x100000  - email
; 0x180000  - write email - 80 step

;[input_pos]- where sender and subject are read - starts at image_end
;[mail_start]- first mail to be read
;[selected_mail]-selected mail 1 

include 'dns.inc'
include 'textbox.inc'

START:

    mov   rdi , 0x180000
  newclear:
    push  rdi
    mov   rcx , 78
    mov   rax , 32
    cld
    rep   stosb
    pop   rdi
    add   rdi , 80
    cmp   rdi , 0x180000 + 80*120
    jb    newclear

    call  get_config

    call  draw_window       ; At first, draw the window

still:

    call  status
    mov   rsi , disconnected
    cmp   [sst],byte 4
    jne   cncd
    mov   rsi , connected     
  cncd:
    call  show_status

    mov   rax , 37
    mov   rbx , 2
    int   0x60
    cmp   rax , 0
    jne   nofetchcheck
    cmp   [sst],byte 4
    jne   nofetchcheck
    inc   qword [scrollchange]
    cmp   dword [scrollchange],dword 50 
    jb    nofetchcheck
    call  check_fetch
  nofetchcheck:

    mov   rax , 23          ; Wait here for event
    mov   rbx , 1 
    int   0x60

    test  rax , 0x1         ; Window redraw
    jnz   window_event
    test  rax , 0x2         ; Keyboard press
    jnz   key_event
    test  rax , 0x4         ; Button press
    jnz   button_event

    cmp   [screen],byte 1
    jne   nomousecheck
    call  check_mouse
  nomousecheck:

    jmp   still


check_mouse:

    mov   rax , 37
    mov   rbx , 2
    int   0x60

    cmp   rax , 0
    je    cml9

    ; check that window is at top

    mov   rax , 26
    mov   rbx , 1
    mov   rcx , 0xf0000
    mov   rdx , 1024
    int   0x60

    mov   r10 , [0xf0000+15*8]

    mov   rax , 26
    mov   rbx , 2
    mov   rcx , 0xf0000
    mov   rdx , 1024
    int   0x60

    imul  r10 , 8
    mov   r10 , [0xf0000+r10]

    mov   rax , 111
    mov   rbx , 1
    int   0x60

    cmp   rax , r10
    jne   cml9

    ;

    mov   rax , 37
    mov   rbx , 1
    int   0x60

    mov   rbx , rax
    shr   rax , 32
    and   rbx , 0xffff

    cmp   rbx , 90
    jb    cml9
    cmp   rbx , 90+12*10
    ja    cml9
    cmp   rax , 18
    jb    cml9 
    cmp   rax , 500
    ja    cml9

    sub   rbx , 90
    mov   rax , rbx
    xor   rdx , rdx
    mov   rbx , 12
    div   rbx

    add   rax , [mail_start]
    mov   [selected_mail], rax

    call  draw_subjects

   cml2:

    mov   rax , 5
    mov   rbx , 1
    int   0x60

    mov   rax , 37
    mov   rbx , 2
    int   0x60

    cmp   rax , 0
    jne   cml2

    cmp   [sst],byte 4
    jne   cml9

    call  fetch_selected_mail 

   cml9:

    ret


get_config:

    ; POP ip

    mov   rax , 112
    mov   rbx , 1
    mov   rcx , pops
    mov   rdx , 99
    mov   r8  , ips1
    int   0x60
    call  get_len

    ; SMTP server

    mov   rax , 112
    mov   rbx , 1
    mov   rcx , smtps
    mov   rdx , 99
    mov   r8  , ips2
    int   0x60
    call  get_len

    ; user

    mov   rax , 112
    mov   rbx , 1
    mov   rcx , users
    mov   rdx , 99 
    mov   r8  , config_user
    int   0x60
    call  get_len

    ; account

    mov   rax , 112
    mov   rbx , 1
    mov   rcx , accounts
    mov   rdx , 99
    mov   r8  , config_account
    int   0x60
    call  get_len

    ret


get_len:

; In : r8      - pointer to string
; Out: [r8-8]  - length of string

    push  rcx

    push  r8

    mov   rcx , 0
  newlens:
    cmp   [r8],byte 32
    jb    lenfound
    inc   r8
    inc   rcx
    jmp   newlens
  lenfound:

    pop   r8

    mov   [r8-8],rcx

    pop   rcx

    ret



get_config_user_pass:  

    mov   rsi , config_user
    mov   rdi , user+5
  gcl0:
    mov   al , [rsi]
    cmp   al , 0
    je    gcl1
    mov   [rdi],al
    inc   rdi
    inc   rsi
    jmp   gcl0
  gcl1:
    mov   [rdi],byte 13
    mov   [rdi+1],byte 10
    mov   [rdi+2],byte 0

    mov   rsi , config_pass
    mov   rdi , pass + 5
  gcl2:
    mov   al , [rsi]
    cmp   al , 0
    je    gcl3
    mov   [rdi] , al
    inc   rdi
    inc   rsi
    jmp   gcl2
  gcl3:
    mov   [rdi],byte 13
    mov   [rdi+1],byte 10
    mov   [rdi+2],byte 0

    ret


window_event:

    call  draw_window
    jmp   still

key_event:

    mov   rax , 2          ; Read the key and ignore
    int   0x60

    cmp   [screen],byte 2
    je    write_email

    jmp   still



write_email:

    test  rbx , 1
    jnz   nodown

    mov   r8 , 'Backspc '
    cmp   rcx , r8
    jne   nobspc2
    cmp   [writex],dword 0
    je    still
    dec   qword [writex]
    mov   rax , [writey]
    add   rax , [scroll3value]
    sub   rax , 300000
    imul  rax , 80
    add   rax , [writex]
    add   rax , 0x180000
    mov   [rax],byte ' '

    call  write_email_draw

    jmp   still
  nobspc2:

    mov   r8 , 'Enter   '
    cmp   rcx , r8
    jne   noent
    cmp   [writey],dword 21
    jb    noel2
    cmp   [scroll3value],dword 300000+100-24  
    jge   still
    inc   qword [scroll3value]
    mov   [writex],dword 0
    call  scroll3
    call  write_email_draw
    jmp   still
  noel2:

    inc   qword [writey]
    mov   [writex],dword 0

    call  write_email_draw

    jmp   still

  noent:

    mov   r8 , 'Up-A    '
    cmp   rcx , r8
    jne   noup
    cmp   dword [writey],dword 0
    je    still
    dec   qword [writey]
    call  write_email_draw
    jmp   still
  noup:
    mov   r8 , 'Down-A  '
    cmp   rcx , r8
    jne   nodown2
    cmp   dword [writey],dword 21
    je    still
    inc   qword [writey]
    call  write_email_draw
    jmp   still
  nodown2:
    mov   r8 , 'Right-A '
    cmp   rcx , r8
    jne   noright
    cmp   dword [writex],dword 78
    je    still
    inc   qword [writex]
    call  write_email_draw
    jmp   still
  noright:
    mov   r8 , 'Left-A  '
    cmp   rcx , r8 
    jne   noleft
    cmp   dword [writex],dword 0
    je    still
    dec   qword [writex]
    call  write_email_draw
    jmp   still
  noleft:

    mov   r8 , 'Home    '
    cmp   rcx , r8
    jne   no_home
    mov   [writex],dword 0
    call  write_email_draw
    jmp   still
  no_home:

  nodown:

    cmp   rbx , 0
    jne   wreml1 

    mov   rax , [writey]
    add   rax , [scroll3value]
    sub   rax , 300000
    imul  rax , 80
    add   rax , [writex]
    add   rax , 0x180000

    mov   [rax],cl       

    cmp   [writex],dword 78
    jae   nowritexi
    inc   qword [writex]
  nowritexi:

    call  write_email_draw

  wreml1:

    jmp   still


write_email_draw:

    mov   rax , [writey]
    add   rax , [scroll3value]
    sub   rax , 300000
    cmp   rax , [reatchedline]
    jb    wedl3
    mov   [reatchedline],rax 
  wedl3:

    mov   rax , 4
    mov   rbx , [scroll3value]
    sub   rbx , 300000
    imul  rbx , 80
    add   rbx , 0x180000
    mov   rcx , 20
    mov   rdx , 170-12+10
    mov   rsi , 0x000000
    mov   r9  , 1
  wedl1:
    push  rax rbx rcx rdx
    mov   rax , 13
    mov   rbx , 20 * 0x100000000 + 6*79
    mov   rcx , rdx
    sub   rcx , 2
    shl   rcx , 32
    add   rcx , 12
    mov   rdx , 0xffffff
    int   0x60
    pop   rdx rcx rbx rax
    int   0x60
    add   rbx , 80
    add   rdx , 12
    cmp   rdx , 158 + 10 + 12*22      
    jb    wedl1

    mov   rax , 38
    mov   rbx , [writex]
    imul  rbx , 6
    add   rbx , 20
    mov   rdx , rbx
    mov   rcx , [writey]
    imul  rcx , 12
    add   rcx , 170-12-2+10
    mov   r8  , rcx
    add   r8  , 10
    mov   r9  , 0x000000
    int   0x60

    ret


connect:

    call  get_config_user_pass

    mov   [mail_start], dword 1
    mov   [selected_mail], dword 1

    mov   [scroll1value],dword 100000
    mov   [scroll2value],dword 200000

    mov   rdi , 0x100000
    mov   rax , 0
    mov   rcx , 80*100
    cld
    rep   stosb
    call  draw_email

    mov   rdi , image_end
    mov   rax , 0
    mov   rcx , 80 * 6000 
    cld
    rep   stosb
    call  draw_subjects

    mov   rsi , opening
    call  show_status

    mov   rsi , ips1
    mov   rdi , pop_ip
    call  get_ip

    cmp   [pop_ip],dword 0
    je    fsl9

    mov   rax , 53
    mov   rbx , 5
    mov   rcx , [localport]
    inc   dword [localport]
    mov   rdx , 110
    mov   rsi , [pop_ip]
    mov   rdi , 1
    int   0x60
    mov   [socket],rax

    mov   r15 , 0
  new_status_wait:
    inc   r15   
    cmp   r15 , 20
    jae   fsl9
    mov   rax , 5
    mov   rbx , 10
    int   0x60
    push  r15
    call  status
    pop   r15
    cmp   [sst],byte 4
    jne   new_status_wait

    mov   [okpos],dword 0x100000
    call  wait_for_ok
    call  draw_email

    ; User

    mov   rsi , user
    call  send
    mov   [okpos],dword 0x100000+80
    call  wait_for_ok
    call  draw_email
    cmp   [0x100000+80],byte '+'
    jne   fsl9

    ; Password

    mov   rsi , pass
    call  send
    mov   [okpos],dword 0x100000+160
    call  wait_for_ok
    call  draw_email
    cmp   [0x100000+160],byte '+'
    jne   fsl9

    ; Stat

    mov   [okpos],dword 0x100000+240
    mov   rsi , stat_command
    call  send
    call  wait_for_ok
    call  draw_email
    cmp   [0x100000+240],byte '+'
    jne   fsl9

    mov   rsi , 0x100000+240
  news:
    inc   rsi
    cmp   [rsi],byte ' '
    jne   news

    mov   rax , 0
  news2:
    inc   rsi
    cmp   [rsi], byte '0'
    jb    news3
    cmp   [rsi], byte '9'
    ja    news3
    imul  rax , 10
    movzx rbx , byte [rsi] 
    sub   rbx , 48
    add   rax , rbx  
    jmp   news2
  news3:

    mov   [emails],rax

    call  scroll1

    ret

  fsl9:

    call  disconnect

    ret


print_inbox_state:

    mov   rax , 4
    mov   rbx , inbox
    mov   rcx , 350
    mov   rdx , 60
    mov   r9  , 1
    mov   rsi , 0x000000
    int   0x60

    mov   rax , 47
    mov   rbx , 6*65536
    mov   rcx , [emails]
    mov   rdx , 410*65536+60
    mov   rsi , 0x000000
    int   0x40

    ret



show_status:

    cmp  rsi , [stat]
    je   stl1

    mov  [stat],rsi

    mov  rax , 13
    mov  rbx , 350 * 0x100000000 + 6*20
    mov  rcx , 50  * 0x100000000 + 22
    mov  rdx , 0xffffff
    int  0x60

    mov  rax , 4
    mov  rbx , rsi
    mov  rcx , 350
    mov  rdx , 50 
    mov  r9  , 1
    mov  rsi , 0x000000
    int  0x60

    call print_inbox_state

  stl1:

    ret



fetch_subjects:

    mov   rsi , reading_subjects
    call  show_status

    mov   rax , [mail_start]
    dec   rax
    imul  rax , 80
    add   rax , image_end
    mov   [headerpos],rax  ; ?      

    mov   rcx , 10 
    mov   r11 , 0

  newheader:

    push  r11
    push  rcx

    mov   rax , r11
    add   rax , [mail_start]

    cmp   rax , [emails]
    ja    no_success

    mov   rdi , top+4
    call  numtostring

    mov   rsi , top
    call  send

  ; mov   [okpos],dword 0xd0000          
  ; mov   rdi , [okpos] 
  ; mov   rcx , 100
  ; mov   rax , 0
  ; cld
  ; rep   stosb
  ; call  wait_for_ok
  ; cmp   [0xd0000],byte '+'           
  ; jne   no_success

    mov   [input_pos],dword 0xf0000             
    call  read_input_header
    call  get_sender
    call  get_subject
;    call  draw_subjects

  no_success:

    pop   rcx
    pop   r11

    inc   r11

    loop  newheader

    call  draw_subjects

    ret



fetch_selected_mail:

    mov   rsi , reading_email
    call  show_status

    mov   rdi , 0xf0000
    mov   rcx , 0x0ffff
    mov   rax , 0
    cld
    rep   stosb

    mov   rax , [selected_mail]
    cmp   rax , [emails]
    ja    fsml8
    mov   rdi , top2+4
    call  numtostring

    mov   rsi , top2
    call  send

    mov   [input_pos],dword 0xf0000
    mov   [endmarker],byte '.'     
    call  read_input_header
    mov   [endmarker],byte '.'

    mov   rsi , 0xf0000
    mov   rdi , 0x100000   
    mov   [scroll2value],dword 200000 

    mov   r12 , 100
  newbes:
    dec   r12
    jz    newbes2
    add   [scroll2value],dword 1
    add   rsi , 80
    mov   rax , '        '
    cmp   [rsi],byte 0        
    jne   newbes
  newbes2:
;   sub   [scroll2value],dword 3

    mov   rsi , 0xf0000
    mov   rcx , 0x7fff
    cld
    rep   movsb
    call  scroll2
    call  draw_email

  fsml8:

    ret



disconnect:

    call  status

    cmp   [sst],byte 0
    je    dcl1

    mov   rsi , disconnecting
    call  show_status

    mov   rsi , quit
    call  send       

    mov   rax , 5
    mov   rbx , 100
    int   0x60

    mov   rax , 53
    mov   rbx , 8
    mov   rcx , [socket]
    int   0x60

    mov   rax , 5
    mov   rbx , 300
    int   0x60

    call  status

    mov   rsi , disconnected
    cmp   [sst],byte 4
    jne   cncd3
    mov   rsi , connected     
  cncd3:
    call  show_status


  dcl1:

    ret


send:

    call  empty_buffer

    mov   rdx , rsi
    dec   rdx
  sel1:
    inc   rdx
    cmp  [rdx],byte 0
    jne   sel1
    sub   rdx , rsi

    mov   rax , 53
    mov   rbx , 7
    mov   rcx , [socket]
    int   0x60

    ret


numtostring:

    add   rdi , 3

    mov   rcx , 4

  ntsl1:

    xor   rdx , rdx
    mov   rbx , 10
    div   rbx

    add   dl , 48
    mov   [rdi],dl

    dec   rdi

    loop  ntsl1

    ret



wait_for_ok:

    mov   rdi , [okpos]   
    add   [okpos],dword 80

    mov   r15 , 0

  wfol0:

    inc   r15
    cmp   r15 , 2000 
    je    wfol1

    mov   rax , 5
    mov   rbx , 1
    int   0x60

  wfol11:

    mov   rax , 53
    mov   rbx , 2
    mov   rcx , [socket]
    int   0x60

    cmp   rax , 0
    je    wfol0

    mov   rax , 53
    mov   rbx , 3
    mov   rcx , [socket]
    int   0x60

    mov   [rdi],bl
    inc   rdi

    push  rbx rdi
 ;  call  draw_subjects
    pop   rdi rbx

    cmp   bl , 10
    je    wfol1

    jmp   wfol11

  wfol1:

    ret


empty_buffer:

    mov   rax , 53
    mov   rbx , 2
    mov   rcx , [socket]
    int   0x60

    cmp   rax , 0
    je    ebl1

    mov   rax , 53
    mov   rbx , 3
    mov   rcx , [socket]
    int   0x60

    jmp   empty_buffer

  ebl1:

    ret



read_input_header:

    mov   rdi , [input_pos]
    push  rdi
    mov   rax , 0
    mov   rcx , 50*80
    cld
    rep   stosb
    pop   rdi

    mov   r10 , 0 ; timeout counter
    mov   r12 , 0 ;

  ril1:

    mov   [input_pos],rdi

    mov   rax , 5
    mov   rbx , 1
    int   0x60

    inc   r10
    cmp   r10 , 1000 ; timeout 10 sec / header
    je    ril2

  ril11:

    mov   rax , 53
    mov   rbx , 2
    mov   rcx , [socket]
    int   0x60

    cmp   rax , 0
    je    ril1

    mov   r10 , 0

    mov   rax , 53
    mov   rbx , 3
    mov   rcx , [socket]
    int   0x60

    cmp   r12 , 0
    jne   nobeginning
    cmp   bl , [endmarker]
    jne   nobeginning
    mov   r14 , 100
  newrest:
    mov   rax , 53
    mov   rbx , 3
    mov   rcx , [socket]
    int   0x60
    dec   r14
    jnz   newrest
    jmp   ril2
  nobeginning:

    cmp   bl , 10
    jne   nolinedown
    mov   r12 , 0
    jmp   ril11
  nolinedown:

    cmp   bl , 13
    jne   nolf
    cmp   r12 , 80
    je    nolf
    sub   rdi , 0xf0000    
    add   rdi , 80
    mov   rax , rdi
    xor   rdx , rdx
    mov   rbx , 80
    div   rbx
    imul  rax , 80
    mov   rdi , rax
    add   rdi , 0xf0000    
    mov   r12 , 0
    jmp   ril11
  nolf:

    cmp   bl , 20 
    jb    ril31
    mov   [rdi],byte bl  
;   cmp   r12 , 79
;   jae   ril3
    inc   rdi
    inc   r12
    jmp   ril31
  ril3:
    cmp   [lflf],byte 1 ; lf only for letter part
    jne   ril31
    mov   r12 , 0
    inc   rdi
  ril31:

    jmp   ril11

  ril2:

 ;; call  draw_subjects

    ret


lflf: dq 0


get_sender:

    mov   rdi , [headerpos]
    mov   rcx , 79
    mov   rax , 32
    cld
    rep   stosb
    mov   al , 0
    stosb

    mov   rsi , 0xf0000   - 80
    mov   rax , 'From'    

  gsel1:

    add   rsi , 80

    cmp   rsi , 0xf0000   +80*100
    ja    gsel2

    cmp   [esi],eax
    jne   gsel1

    add   rsi , 6
    cmp   [rsi],byte 31
    ja    noquote
    inc   rsi
  noquote:

    mov   rdi , [headerpos] 
    add   rdi , 7
    mov   r10 , rsi
  gsel4:
    mov   al , [rsi]
    cmp   rsi , r10
    je    nobc
    cmp   al , '<'
    je    gsel5
  nobc:
    cmp   al , 31
    jb    gsel5
    mov   [rdi], al
    inc   rsi
    inc   rdi
    mov   rax , [headerpos]
    add   rax , 29
    cmp   rdi , rax
    jae   gsel5
    jmp   gsel4
  gsel5:

    mov   rsi , [headerpos]
  zcheck:
    cmp   [rsi],byte 0
    jne   noz
    mov   [rsi],byte 32
  noz:
    inc   rsi
    mov   rdi , [headerpos]
    add   rdi , 50
    cmp   rsi , rdi           
    jb    zcheck 

  ; call  draw_subjects

  gsel2:

    ret 


get_subject:

    mov   rsi , 0xf0000   - 80
    mov   rax , 'Subject:'

  gsl1:

    add   rsi , 80

    cmp   rsi , 0xf0000   +80*100
    ja    gsl2

    cmp  [rsi], rax 
    jne   gsl1   

    add   rsi , 9

    mov   rdi , [headerpos]    
    add   rdi , 32
    mov   rcx , 47
    cld
    rep   movsb

  ; call  draw_subjects

  gsl2:

    add   [headerpos],dword 80

    ret 


status:

    mov   rax , 53
    mov   rbx , 6
    mov   rcx , [socket]
    int   0x60

    mov   [sst],rax

    ret




button_event:

    mov   rax , 0x11
    int   0x60

    ; rax = status
    ; rbx = button id

    cmp   rbx , 1001
    jne   notextboxentry1
    call  disconnect
    mov   r14 , textbox1
    call  read_textbox
    jmp   still
  notextboxentry1:
    cmp   rbx , 1002
    jne   notextboxentry2
    call  disconnect
    mov   r14 , textbox2
    call  read_textbox
    jmp   still
  notextboxentry2:
    cmp   rbx , 1003
    jne   notextboxentry3
    call  disconnect
    mov   [textbox3+5*8],dword 0
    mov   [textbox3+6*8],dword 0
    mov   r14 , textbox3
    call  read_textbox
    mov   rsi , textbox3+6*8
    mov   rdi , config_pass
    mov   rcx , 50
    cld
    rep   movsb
    mov   [textbox3+5*8],dword 0
    mov   rax , '********'
    mov   [textbox3+6*8],rax
    mov   rax , 0
    mov   [textbox3+7*8],rax
    mov   r14 , textbox3
    call  draw_textbox
    jmp   still
  notextboxentry3:
    cmp   rbx , 1004
    jne   notextboxentry4
    call  disconnect
    mov   r14 , textbox4
    call  read_textbox
    jmp   still
  notextboxentry4:
    cmp   rbx , 1005
    jne   notextboxentry5
    call  disconnect
    mov   r14 , textbox5
    call  read_textbox
    jmp   still
  notextboxentry5:
    cmp   rbx , 1011
    jne   notextboxentry11
    call  disconnect
    mov   r14 , textbox11
    call  read_textbox
    jmp   still
  notextboxentry11:
    cmp   rbx , 1012
    jne   notextboxentry12
    call  disconnect
    mov   r14 , textbox12
    call  read_textbox
    jmp   still
  notextboxentry12:
 
    cmp   rbx , 0x10000001
    jne   no_application_terminate_button
    call  disconnect
    mov   rax , 0x200
    int   0x60
  no_application_terminate_button:

    cmp   rbx , 0x106
    jne   no_application_terminate_menu
    call  disconnect
    mov   rax , 0x200
    int   0x60
  no_application_terminate_menu:

    cmp   rbx , 300000
    jb    no_scroll3
    cmp   rbx , 399999
    ja    no_scroll3
    mov   [scroll3value],rbx
    call  scroll3
    call  write_email_draw
    jmp   still
  no_scroll3:

    cmp   rbx , 200000
    jb    no_scroll2
    cmp   rbx , 299999
    ja    no_scroll2
    mov   [scroll2value],rbx
    call  scroll2
    call  draw_email
    jmp   still
  no_scroll2:

    cmp   rbx , 100000
    jb    no_scroll1
    cmp   rbx , 199999
    ja    no_scroll1
    mov   [scroll1value],rbx
    sub   rbx , 100000
    imul  rbx , 10
    inc   rbx
    mov   [mail_start],rbx
    call  scroll1
    call  draw_subjects
    jmp   still
  no_scroll1:

    cmp   rbx , 11
    jb    noscreenstate
    cmp   rbx , 15
    ja    noscreenstate
    sub   rbx , 10
    cmp   rbx , [screen]
    je    still
    mov   [screen],rbx
    call  draw_window
    cmp   [screen],byte 1
    jne   norec
    cmp   [sst],byte 4
    je    norec
    call  connect
  norec:
    jmp   still
  noscreenstate:

    cmp   rbx , 51
    jne   no_send_button

    call  send_email

  no_send_button:

    jmp   still



send_email:

    mov   rsi , sending
    call  show_user

    ; disconnect inbox

    call  disconnect

    mov   rax , 5
    mov   rbx , 50
    int   0x60

    call  status
    mov   rsi , disconnected
    call  show_status

    mov   rsi , ips2
    mov   rdi , smtp_ip
    call  get_ip

    cmp   [smtp_ip],dword 0
    je    smtpipfail

    ; Open socket

    mov   rax , 53
    mov   rbx , 5
    mov   rcx , [localport]
    inc   dword [localport]
    mov   rdx , 25       
    mov   rsi , [smtp_ip]
    mov   rdi , 1
    int   0x60
    mov   [send_socket],rax

    mov   rax , 5
    mov   rbx , 100
    int   0x60

    ; Helo mycomputer

    mov   rsi , helo
    call  send_email_data
    call  wait_for_send_result
    cmp   [result],dword '250 '     
    jne   sel99

    mov   rsi , header
    call  show_user

    ; Mail from:< > 

    mov   rsi , mail_from
    call  send_email_data
    mov   rsi , config_account
    call  send_email_data
    mov   rsi , close_add
    call  send_email_data
    call  wait_for_send_result
    cmp   [result],dword '250 '
    jne   sel99

    ; Rcpt to:< >

    mov   rsi , rcpt_to
    call  send_email_data
    mov   rsi , config_sendto
    call  send_email_data
    mov   rsi , close_add
    call  send_email_data
    call  wait_for_send_result
    cmp   [result],dword '250 '
    jne   sel99

    ; Data

    mov   rsi , data_send
    call  send_email_data
    call  wait_for_send_result
    cmp   [result],dword '354 '
    jne   sel99

    mov   rsi , body
    call  show_user

    ; Subject:

    mov   rsi , data_subject
    call  send_email_data
    mov   rsi , config_subject
    call  send_email_data
    mov   rsi , send_lf
    call  send_email_data

    mov   rax , 5
    mov   rbx , 20 
    int   0x60

    ; To:    

    mov   rsi , data_to
    call  send_email_data
    mov   rsi , config_sendto
    call  send_email_data
    mov   rsi , send_lf
    call  send_email_data

    ; lf

    mov   rsi , send_lf
    call  send_email_data
    mov   rsi , send_lf
    call  send_email_data

    mov   rax , 5
    mov   rbx , 20 
    int   0x60

    ; Email body       

    mov   rdi , 0x180000
    mov   rax , 0

  sendnewline:

    push  rax
    push  rdi

    mov   rsi , rdi             
    call  send_email_data

    mov   rax , 5
    mov   rbx , 10
    int   0x60

    mov   rsi , send_lf
    call  send_email_data

    pop   rdi
    pop   rax

    add   rdi , 80

    inc   rax

    cmp   rax , 110
    ja    nomorelines

    cmp   rax , [reatchedline]
    jbe   sendnewline

  nomorelines:

    ; '.'

    mov   rsi , data_end
    call  send_email_data
    call  wait_for_send_result
    cmp   [result],dword '250 '
    jne   sel99

  succ:

    ; Quit

    mov   rsi , send_quit
    call  send_email_data

    ; Success

    mov   rax , 5
    mov   rbx , 20
    int   0x60

    call  close_send_socket

    mov   rsi , success
    call  show_user

    ret

  sel99:

    ; Fail

    mov   rsi , send_quit
    call  send_email_data

    mov   rax , 5
    mov   rbx , 20
    int   0x60

    call  close_send_socket

  smtpipfail:

    mov   rsi , fail
    call  show_user

    ret


close_send_socket:

    mov   rax , 53
    mov   rbx , 8
    mov   rcx , [send_socket]
    int   0x60

    ret


send_email_data:

    mov   rax , '        '
    mov   [result],rax

    ; check socket_state

    mov   rax , 53
    mov   rbx , 6
    mov   rcx , [send_socket]
    int   0x60

    cmp   rax , 4
    jne   skipsend  

    ; read all data from socket

  read_more_data:

    mov   rax , 53
    mov   rbx , 2
    mov   rcx , [send_socket]
    int   0x60

    cmp   rax , 0
    je    no_data_in_buffer

    mov   rax , 53
    mov   rbx , 3
    mov   rcx , [send_socket]
    int   0x60

    jmp   read_more_data

  no_data_in_buffer:

    ; send data

    mov   rdx , rsi
    dec   rdx
  sdsl1:
    inc   rdx
    cmp   [rdx],byte 0
    jne   sdsl1
    sub   rdx , rsi

    and   rdx , 0xff
    cmp   rdx , 0
    je    skipsend

    mov   rax , 53
    mov   rbx , 7
    mov   rcx , [send_socket] 
    int   0x60

    ; rsi = pointer to asciiz 

 ;; call  show_user

  skipsend:

    mov   rax , 5
    mov   rbx , 5 
    int   0x60

    ret


wait_for_send_result:

    ; read sockets first 4 bytes to result+ until 13,10

    mov   rdi , 0
    mov   r15 , 0

  wait_for_data:

    mov   rax , 5
    mov   rbx , 1
    int   0x60

    ; Timeout 10 seconds
                    
    inc   r15
    cmp   r15 , 200 
    jae   nomoredata

  wfdl1:

    mov   rax , 53
    mov   rbx , 2
    mov   rcx , [send_socket]
    int   0x60

    cmp   rax , 0
    je    wait_for_data

    mov   r15 , 0

    mov   rax , 53
    mov   rbx , 3
    mov   rcx , [send_socket]
    int   0x60

    cmp   rdi , 4
    jae   nodatabuf
    mov   [result+rdi],bl
    inc   rdi
  nodatabuf:

    cmp   bl , 10
    je    nomoredata

    jmp   wfdl1

  nomoredata:

    ret



show_user:

    mov   rax , 13
    mov   rbx , 350 * 0x100000000 + 6*26
    mov   rcx , 091 * 0x100000000 + 12
    mov   rdx , 0xffffff
    int   0x60

    mov   rax , 4
    mov   rbx , rsi
    mov   rcx , 350
    mov   rdx , 091
    mov   rsi , 0x000000
    mov   r9  , 1
    int   0x60

    ret


draw_window:

    mov   rax , 0xC                          ; Beginning of window draw
    mov   rbx , 0x1
    int   0x60

    mov   rax , 0x0                          ; Draw window
    mov   rbx , 0x0000004000000000+60+6*80   ; x start & size
    mov   rcx , 0x00000030000001C0           ; y start & size
    mov   rdx , 0x0000000000FFFFFF           ; type    & border color  
    mov   r8  , 0x0000000000000001           ; draw flags
    mov   r9  , window_label                 ; 0 or label - asciiz
    mov   r10 , menu_struct                  ; 0 or pointer to menu struct
    int   0x60

    cmp   [screen],byte 1
    jne   noscreen1
    call  inbox_screen
  noscreen1:

    cmp   [screen],byte 2
    jne   noscreen2
    call  send_screen
  noscreen2:

    cmp   [screen],byte 3
    jne   noscreen3
    call  setup_screen
  noscreen3:

    mov   rax , 8
    mov   rbx , 15 * 0x100000000 + 60
    mov   rcx , 50 * 0x100000000 + 19
    mov   rdx , 11
    mov   r8  , 0
    mov   r9  , inbox_text
    int   0x60
    mov   rbx , 75 * 0x100000000 + 60
    inc   rdx
    mov   r9  , send_text
    int   0x60
    mov   rbx , 135 * 0x100000000 + 60
    inc   rdx
    mov   r9  , settings_text
    int   0x60

    mov   rsi , [stat]
    mov   [stat], dword 1
    call  show_status

    mov   rax , 0xc
    mov   rbx , 2
    int   0x60

    ret


inbox_screen:

    call  scroll1
    call  scroll2

    call  draw_subjects
    call  draw_email

 ;  mov   rsi , [stat]
 ;  mov   [stat],dword 1
 ;  call  show_status

    ret


send_screen:

    mov   rax , 4
    mov   rbx , send_text_address
    mov   rcx , 25
    mov   rdx , 95+4 - 4
    mov   rsi , 0x000000
    mov   r9  , 1
  sendl1:
    int   0x60
    add   rdx , 20
    add   rbx , 30
    cmp   [rbx],byte 'x'
    jne   sendl1

    mov   rax , 4
    mov   rcx , 24+6*10;6*2
    mov   rdx , 99-4;20*2
    mov   rbx , config_account
    mov   r9  , 1
    mov   rsi , 0x000000
    int   0x60

    mov   rbx , 16
    mov   rcx , 86
    mov   rdx , 522
    mov   r8  , 138+15
    call  rectangle2

    mov   rbx , 16
    mov   rcx , 153+10
    mov   rdx , 522
    mov   r8  , 433
    call  rectangle2

    call  write_email_draw

    mov   rax , 38
    mov   rbx , 315
    mov   rcx , 86
    mov   rdx , 315
    mov   r8  , 138+15
    mov   r9  , 0x000000
    int   0x60

    mov   rax , 8
    mov   rbx , 344 * 0x100000000 + 149
    mov   rcx , (103+6) * 0x100000000 + 17 
    mov   rdx , 51
    mov   r8  , 0
    mov   r9  , ready_for_send
    int   0x60

    call  scroll3

    mov   r14 , textbox11
    call  draw_textbox
    mov   r14 , textbox12
    call  draw_textbox

    ret



rectangle2:

    mov   rax , 38
    mov   r9  , 0x000000

    push  rdx
    mov   rdx , rbx
    int   0x60
    pop   rdx
    push  rbx
    mov   rbx , rdx
    int   0x60
    pop   rbx 
    push  rcx
    mov   rcx , r8
    int   0x60
    pop   rcx
    push  r8
    mov   r8  , rcx
    int   0x60
    pop   r8

    ret



setup_screen:

    mov   rbx , 16
    mov   rcx , 86
    mov   rdx , 522
    mov   r8  , 433
    call  rectangle2

    mov   rax , 4
    mov   rbx , setup_text
    mov   rcx , 25
    mov   rdx , 95 
    mov   r9  , 1
    mov   rsi , 0x000000

  setl1:

    int   0x60

    add   rbx , 79
    add   rdx , 20
    cmp   [rbx],byte 'x'
    jne   setl1

;    mov   r14 , [pop_ip]
;    mov   r15 , 95+12*2
;    call  print_ip
;    mov   r14 , [smtp_ip]
;    mov   r15 , 95+12*8
;    call  print_ip

    mov   rax , 4
    mov   rbx , config_user
    mov   rcx , 25+17*6
    mov   rdx , 95+3*12
    mov   rsi , 0x000000
    mov   r9  , 1
;   int   0x60
    push  rdx
    mov   rbx , ips1
    sub   rdx , 12
;   int   0x60
    mov   rbx , ips2
    add   rdx , 12*6
;   int   0x60
    pop   rdx

    mov   rbx , config_real
    add   rdx , 12*7
;   int   0x60
    mov   rbx , config_account
    sub   rdx , 12
;   int   0x60

    mov   rax , 8
    mov   rbx , 310 * 0x100000000 + 11
    mov   rcx , 116 * 0x100000000 + 11
    mov   rdx , 22
    mov   r8  , 0
    mov   r9  , arrow
    mov   r10 , 12 * 0x100000000
  newbutton:
;   int   0x60
    cmp   rdx , 24
    jne   nordx24
    add   rdx , 3  
    add   rcx , r10
    add   rcx , r10
    add   rcx , r10
  nordx24:
    add   rcx , r10
    inc   rdx
    cmp   rdx , 21+8
    jbe   newbutton

    mov   r14 , textbox1
    call  draw_textbox
    mov   r14 , textbox2
    call  draw_textbox

    mov   r14 , textbox3
    call  draw_textbox
    mov   r14 , textbox4
    call  draw_textbox
    mov   r14 , textbox5
    call  draw_textbox

    ret



print_ip:

    mov   rax , 47
    mov   rbx , 3*65536
    mov   rdx , (25+17*6)*65536
    add   rdx , r15
    mov   rsi , 0x000000
    mov   r10 , 4
  pipl1:
    mov   rcx , r14
    and   rcx , 0xff
    int   0x40
    add   rdx , (4*6)*65536
    shr   r14 , 8
    dec   r10 
    jnz   pipl1

    ret


check_fetch:  

    mov   rax , [mail_start]
    dec   rax
    imul  rax , 80
    add   rax , image_end+78
    cmp   [rax],byte 0 
    jne   nofetch
    ; print loading
    push  rax
    call  fetch_subjects
    call  draw_subjects
    pop   rax
    mov   [rax],byte 1
  nofetch:

    ret


draw_subjects:

    mov   rcx , [mail_start]
    mov   r14 , 0
    mov   rdx , 26 * 65536+90
    mov   rsi , 0x000000
  dsl2:
    mov   rax , 47
    mov   rbx , 4*65536

    push  r10
    push  rax rbx rcx rdx
    mov   r12 , rcx
    mov   rax , 13
    mov   rbx , 18*0x100000000+81*6+5
    mov   rcx , rdx
    and   rcx , 0xffff
    sub   rcx , 2
    shl   rcx , 32
    add   rcx , 1 
    mov   rdx , 0xffffff
    mov   rsi , 0x000000
    mov   r10 , 12
    cmp   r12 , [selected_mail]
    jne   nosel
    mov   rdx , 0xcccccc
;;; mov   rdx , 0x66aadd
;;; mov   rsi , 0xffffff
  nosel:
    int   0x60

    cmp   rdx , 0xffffff
    je    nocsub
;;; sub   rdx , 0x030303
  nocsub:
    mov   r12 , 0x100000000
    add   rcx , r12
    dec   r10
    jnz   nosel

    pop   rdx rcx rbx rax
    pop   r10

    int   0x40
    add   rdx , 12
    inc   rcx
    inc   r14
    cmp   r14 , 10
    jb    dsl2

    mov   rax , 4
    mov   rbx , [mail_start]
    dec   rbx
    imul  rbx , 80
    add   rbx , image_end   ; ????
    mov   rcx , 20
    mov   rdx , 90 
    mov   r9  , 1
    mov   r12 , [mail_start]
    mov   r15 , 10
  dsl1:    
    mov   rsi , 0x000000
    cmp   r12 , [selected_mail]
    jne   nosel2
;;; mov   rsi , 0xffffff
  nosel2:
    int   0x60
    add   rdx , 12
    add   rbx , 80
    inc   r12
    dec   r15
    jnz   dsl1

    ret


draw_email:

    mov   rax , 4
    mov   rbx , [scroll2value]
    sub   rbx , 200000
    imul  rbx , 80
    add   rbx , 0x100000
    mov   rcx , 20
    mov   rdx , 230
    mov   r9  , 1
    mov   rsi , 0x000000
    mov   r15 , 17
  del1:
    mov   r14b , [rbx+80]
    mov   [rbx+80],byte 0
    push  rax rbx rcx rdx
    mov   rax , 13
    mov   rbx , 20*0x100000000 + 6*80
    mov   rcx , rdx
    shl   rcx , 32
    add   rcx , 11      
    mov   rdx , 0xffffff
    int   0x60
    pop   rdx rcx rbx rax
    int   0x60
    mov   [rbx+80],r14b
    add   rdx , 12
    add   rbx , 80
    dec   r15
    jnz   del1

    ret


scroll1:

    mov   rax , [emails]
    cmp   rax , 0
    je    scl1
    dec   rax
  scl1:
    xor   rdx , rdx
    mov   rbx , 10
    div   rbx
    inc   rax
    mov   rdx , rax

    mov   rax , 113
    mov   rbx , 1
    mov   rcx , 100000
    mov   r8  , [scroll1value]
    mov   r9  , 510
    mov   r10 , 87
    mov   r11 , 12*10+1  
    int   0x60

    mov   [scrollchange],dword 0

    call  rectangle

    ret


scroll2:

    mov   rax , 113
    mov   rbx , 1
    mov   rcx , 200000
    mov   rdx , 200
    mov   r8  , [scroll2value]
    mov   r9  , 510
    mov   r10 , 227
    mov   r11 , 205
    int   0x60

    call  rectangle

    ret


scroll3:

    mov   rax , 113
    mov   rbx , 1
    mov   rcx , 300000
    mov   rdx , 100-23
    mov   r8  , [scroll3value]
    mov   r9  , 510
    mov   r10 , 154+10
    mov   r11 , 278-10
    int   0x60

    ret


rectangle:

    mov   rcx , r10
    dec   rcx
    mov   r8  , rcx

    mov   rax , 38
    mov   rbx , 16
    mov   rdx , 510+12
    mov   r9  , 0x000000
    int   0x60

    push  rcx

    add   rcx , r11
    add   rcx , 2
    mov   r8  , rcx
    int   0x60

    mov   rdx , rbx
    pop   rcx
    int   0x60

    ret


; Data area

window_label:

    db    'MAIL',0        ; Window label

inbox_text:

    db    'INBOX',0

send_text:

    db    'SEND',0

settings_text:

    db    'SETUP',0       

scroll1value:

    dq    100000

scroll2value:

    dq    200000

scroll3value:

    dq    300000

screen:

    dq    2

smtp_ip:

    db    192,168,0,97
    dd    0

send_text_address:

    db    'Sender:                      ',0
    db    'Send to:                     ',0
    db    'Subject:                     ',0
    db    'x'

textbox1:

    dq    0         ; Type
    dq    100       ; X position
    dq    220       ; X size
    dq    130-20    ; Y position
    dq    1001      ; Button ID
    dq    0         ; Current text length
ips1:
    times 150 db 0  ; Text

textbox2:

    dq    0         ;     
    dq    100       ;
    dq    220       ;
    dq    150-20    ;
    dq    1002      ;
    dq    0         ;
config_user:
    times 150 db 0  ;

textbox3:

    dq    0         ;     
    dq    100       ;
    dq    220       ;
    dq    170-20    ;
    dq    1003      ;
    dq    0         ;
    db    '********'
    times 150 db 0  ;

config_pass: times 150 db 0


textbox4:

    dq    0         ;     
    dq    100       ;
    dq    220       ;
    dq    250-40    ;
    dq    1004      ;
    dq    0         ;
ips2:
    times 150 db 0  ;

textbox5:

    dq    0         ;     
    dq    100       ;
    dq    220       ;
    dq    270-40    ;
    dq    1005      ;
    dq    0         ;
config_account:
    times 150 db 0  ;


textbox11:

    dq    0         ; Type
    dq    80        ; X position
    dq    220       ; X size
    dq    94+20-4   ; Y position
    dq    1011      ; Button ID
    dq    0         ; Current text length
config_sendto:
    times 150 db 0  ; Text

textbox12:

    dq    0         ;     
    dq    80        ;
    dq    220       ;
    dq    114+20-4  ;
    dq    1012      ;
    dq    0         ;
config_subject:
    times 150 db 0  ;

setup_text:

    db    'Incoming                               '
    db    '                                       ',0
    db    'POP server:                            '  
    db    '                                       ',0
    db    'Username:                              '  
    db    '                                       ',0
    db    'Password:        ********              '  
    db    '                                       ',0
    db    '                                       '
    db    '                                       ',0
    db    'Outgoing                               ' 
    db    '                                       ',0
    db    'SMTP server:                           '
    db    '                                       ',0
    db    'Account:                               '
    db    '                                       ',0
    db    '                                       '
    db    '                                       ',0
    db    'Default values ( except password ) in C'                       
    db    'onfig.mnt.                             ',0
    db    'After changing values, press INBOX to r'
    db    'econnect.                              ',0
    db    'x'


menu_struct:               ; Menu Struct

    dq   0                 ; Version

    dq   0x100             ; Start value of ID to return ( ID + Line )

                           ; Returned when menu closes and
                           ; user made no selections.

    db   0,'FILE',0        ; ID = 0x100 + 1
    db   1,'New',0         ; ID = 0x100 + 2
    db   1,'Open..',0      ; ID = 0x100 + 3
    db   1,'Save..',0      ; ID = 0x100 + 4
    db   1,'-',0           ; ID = 0x100 + 5
    db   1,'Quit',0        ; ID = 0x100 + 6

    db   0,'HELP',0        ; ID = 0x100 + 7
    db   1,'Contents..',0  ; ID = 0x100 + 8
    db   1,'About..',0     ; ID = 0x100 + 9

    db   255               ; End of Menu Struct


pops:     db 'email_pop_server  ',0  
users:    db 'email_pop_user    ',0 
smtps:    db 'email_smtp_server ',0
accounts: db 'email_smtp_user   ',0

writex: dq 0
writey: dq 0

reatchedline: dq 1

reading_email: db 'Reading email..',0

mail_start: dq 1    

localport: dq 23543

opening db 'Opening..',0

inbox: db 'Messages:',0 

emails: dq 0

stat_command: db 'stat',13,10,0

disconnected: db 'Disconnected',0

connected:    db 'Connected',0

stat: dq 1           

reading_subjects: db 'Reading headers..',0

disconnecting: db 'Disconnecting',0

okpos: dq 0xe0000         

input_pos: dq image_end

endmarker: db '.'

headerpos: dq image_end

sst: dq 0

user: db 'User '                   
      times 120 db 0
pass: db 'Pass '                  
      times 120 db 0

lf:   db 13,10,0

top:  db 'Top 4496 0',13,10,0
top2: db 'Top 4496 100',13,10,0
retr: db 'Retr 4495',13,10,0

quit: db 'Quit',13,10,0

pop_ip: db 192,168,0,99
        dd 0

socket: dq 0x0

sending: db 'Connecting..',0
success: db 'Success',0
fail:    db 'Fail',0

header:  db 'Sending header..',0
body:    db 'Sending body..',0
            
result:    dq 0

send_socket: dq 0

helo:      db 'Helo mycomputer',13,10,0

mail_from: db 'Mail from:<',0 
rcpt_to:   db 'Rcpt to:<',0 
close_add: db '>',13,10,0
data_send: db 'Data',13,10,0

data_subject: db 'Subject: ',0
data_to:      db 'To: ',0
data_end:     db 13,10,13,10,13,10,'.',13,10,0

send_lf:   db 13,10,0

send_quit: db 'quit',13,10,0
            
ready_for_send: db 'EMAIL READY FOR SEND',0
            
arrow: db '<',0
            
selected_mail: dq 1
            
scrollchange: dq 0x0


I_END:


estring:         times 100 db ?

config_real:     times 200 db ?

config_reply:    times 200 db ?

image_end:




























