;
;   64 bit Menuet usb info
;
;   Compile with FASM 1.60 or above
;

use64

    org   0x0

    db    'MENUET64'              ; Header identifier
    dq    0x01                    ; Version
    dq    START                   ; Start of code
    dq    image_end               ; Size of image
    dq    0x100000                ; Memory for app
    dq    0xffff0                 ; Esp
    dq    0x00                    ; Prm 
    dq    0x00                    ; Icon


START:

    call  draw_window       ; At first, draw the window

still:

    mov   rax , 23          ; Wait here for event
    mov   rbx , 20
    int   0x60

    test  rax , 0x1         ; Window redraw
    jnz   window_event
    test  rax , 0x2         ; Keyboard press
    jnz   key_event
    test  rax , 0x4         ; Button press
    jnz   button_event

    call  display_dev

    jmp   still

window_event:

    call  draw_window
    jmp   still

key_event:

    mov   rax , 0x2        ; Read the key and ignore
    int   0x60

    jmp   still

button_event:

    mov   rax , 0x11
    int   0x60

    ; rax = status
    ; rbx = button id

    cmp   rbx , 0x10000001
    jne   no_application_terminate_button
    mov   rax , 0x200
    int   0x60
  no_application_terminate_button:

    cmp   rbx , 0x106
    jne   no_application_terminate_menu
    mov   rax , 0x200
    int   0x60
  no_application_terminate_menu:

    jmp   still


draw_window:

    mov   rax , 0xC                          ; Beginning of window draw
    mov   rbx , 0x1
    int   0x60

    mov   rax , 0x0                          ; Draw window
    mov   rbx , 0x0000008500000000 + 519     ; x start & size
    mov   rcx , 0x0000004500000000 + 317     ; y start & size
    mov   rdx , 0x0000000000FFFFFF           ; type    & border color  
    mov   r8  , 0x0000000000000001           ; draw flags
    mov   r9  , window_label                 ; 0 or label - asciiz
    mov   r10 , menu_struct                  ; 0 or pointer to menu struct
    int   0x60

    mov   [checksum],dword 0xff
    call  display_dev

    mov   rax , 0xC                          ; End of window draw
    mov   rbx , 0x2
    int   0x60

    ret

checksum: dq 0x0

display_dev:

    ; Prescan for change

    mov   rcx , 1

    mov   r15 , 0

  newprescan:

    mov   rax , 127
    mov   rbx , 2
    int   0x60

    add   r15 , rax

    mov   rax , 127
    mov   rbx , 3
    mov   rdx , image_end
    mov   r8  , 32
    int   0x60

    add   r15 , [image_end]

    mov   rax , 127
    mov   rbx , 4
    mov   rdx , image_end
    mov   r8  , 32
    int   0x60

    add   r15 , [image_end]

    mov   rax , 127
    mov   rbx , 5
    mov   rdx , image_end
    mov   r8  , 32
    int   0x60

    add   r15 , [image_end]

    imul  r15 , rcx

    add   rcx , 1
    cmp   rcx , 16
    jbe   newprescan

    cmp   r15 , [checksum]
    je    no_device_display

    mov   [checksum],r15

    ; Display information

    mov   rax , 127
    mov   rbx , 1
    int   0x60

    mov   r15 , 'Disabled'
    cmp   rax , 1
    jne   usbs1
    mov   r15 , 'Enabled '
  usbs1:
    cmp   rax , 1
    jbe   usbs2
    xor   rdx , rdx
    mov   rbx , 10
    div   rbx
    mov   ah , dl
    mov   r15 , 'Fail(00)'
    shl   rax , 40
    add   r15 , rax
  usbs2:


    mov  [text+11],r15

    mov   rax , 4            ; Display text
    mov   rbx , text
    mov   rcx , 21
    mov   rdx , 0x36
    mov   rsi , 0x0
    mov   r9  , 0x1
    mov   r8  , 0x2
    int   0x60
    add   rbx , 50
    add   rdx , 20
    int   0x60
    add   rbx , 50
    add   rdx , 20
    int   0x60

;;;    cmp   r15b, 'E'   
;;;    jne   no_device_display

    ; Display port information

    mov   rcx , 1

  new_device:

    push  rcx

    ; Default device info

    mov   rsi , device_info_default
    mov   rdi , device_info
    mov   rcx , device_info_default_end - device_info_default
    cld
    rep   movsb

    ; Port number

    mov   rax , [rsp]
    xor   rdx , rdx
    mov   rbx , 10
    div   rbx
    mov   [device_info],al
    mov   [device_info+1],dl
    add   [device_info],word 0x3030

    ; USB version

    mov   rax , 127
    mov   rbx , 2
    mov   rcx , [rsp]
    int   0x60

    cmp   rax , 0
    je    doclear

    mov   rbx , 'Reading '
    mov   [device_info+5],rbx
    mov   bl , ' '
    mov   [device_info+5+8],bl

    cmp   rax , 3
    je    doclear

    cmp   rax , 1
    jne   nousb1x
    mov   rbx , 'Disabled'
    mov   [device_info+5],rbx
    mov   [device_info+13],byte ' '
    mov   [device_info+17],dword 'USB '
    mov   [device_info+21],dword '1.X '
    jmp   doclear
  nousb1x:

    cmp   rax , 2    
    jne   nousb2
    mov   rbx , 'Connecte'
    mov   [device_info+5],rbx
    mov   [device_info+13],byte 'd'
    mov   [device_info+17],dword 'USB '
    mov   [device_info+21],dword '2.0 '
  nousb2:

    ; Manufacturer

    mov   rax , 127
    mov   rbx , 3
    mov   rcx , [rsp]
    mov   rdx , device_info+37
    mov   r8  , 32
    int   0x60

    mov   rdx , device_info+36
  news:
    inc   rdx
    cmp   [rdx],byte 0
    ja    news

    mov   [rdx],word ': '
    add   rdx , 2

    ; Product

    mov   rax , 127
    mov   rbx , 4
    mov   rcx , [rsp]
    mov   r8  , 32
    int   0x60

    ; Path

    mov   rax , 127
    mov   rbx , 5
    mov   rcx , [rsp]
    mov   rdx , device_info+27
    mov   r8  , 1
    int   0x60
    add   byte [device_info+27],'-'
    cmp   byte [device_info+27],'-'
    je    doclear
    mov   rax , 127
    mov   rbx , 5
    mov   rcx , [rsp]
    mov   rdx , device_info+27
    mov   r8  , 7
    int   0x60

  doclear:

    ; Change ascii zeroes to ' '

    mov   rdi , device_info
    mov   rcx , device_info_default_end - device_info_default-3
  asciizero:
    cmp   [rdi],byte 0
    jne   nozero
    mov   [rdi],byte ' '
  nozero:
    inc   rdi
    loop  asciizero

    ; Clear

    mov   rax , 13
    mov   rbx , 19 shl 32 + 6*80 +2
    mov   rcx , [rsp]
    imul  rcx , 12
    add   rcx , 97
    shl   rcx , 32
    add   rcx , 11
    mov   rdx , 0xf0f0f0
    int   0x60

    mov   rax , 4          
    mov   rbx , device_info
    mov   rcx , 21
    mov   rdx , [rsp]
    imul  rdx , 12
    add   rdx , 99
    mov   rsi , 0x0
    mov   r9  , 0x1
    mov   r8  , 0x2
    int   0x60

    pop   rcx

    inc   rcx 
    cmp   rcx , 16
    jbe   new_device

  no_device_display:

    ret



; Data area

window_label:

    db    'USB 2.0 DEVICES',0     ; Window label

device_info:  

    db    '00   No Device  ---  ---  ---                             '
    db    '                                                          ',0

device_info_default:

    db    '00   No Device                                            '
    db    '                                                          ',0


device_info_default_end:


text:

    db    'USB state:                                       ',0
    db    'Define USB state at Config.mnt and reboot.       ',0
    db    'Port Status      Protocol   Path     Manufacturer and product',0

menu_struct:               ; Menu Struct

    dq   0                 ; Version

    dq   0x100             ; Start value of ID to return ( ID + Line )

                           ; Returned when menu closes and
                           ; user made no selections.

    db   0,'FILE',0        ; ID = 0x100 + 1
    db   1,'New',0         ; ID = 0x100 + 2
    db   1,'Open..',0      ; ID = 0x100 + 3
    db   1,'Save..',0      ; ID = 0x100 + 4
    db   1,'-',0           ; ID = 0x100 + 5
    db   1,'Quit',0        ; ID = 0x100 + 6

    db   0,'HELP',0        ; ID = 0x100 + 7
    db   1,'Contents..',0  ; ID = 0x100 + 8
    db   1,'About..',0     ; ID = 0x100 + 9

    db   255               ; End of Menu Struct

image_end:

